"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.globModels = exports.DataSourceManager = void 0;
/**
 * 数据源管理器实现
 */
const extend_1 = require("../util/extend");
const error_1 = require("../error");
const glob_1 = require("@midwayjs/glob");
const path_1 = require("path");
const decorator_1 = require("@midwayjs/decorator");
const DEFAULT_PATTERN = ['**/**.ts', '**/**.js'];
class DataSourceManager {
    constructor() {
        this.dataSource = new Map();
        this.options = {};
        this.modelMapping = new WeakMap();
    }
    async initDataSource(options, appDir) {
        this.options = options;
        if (options.dataSource) {
            for (const dataSourceName in options.dataSource) {
                const dataSourceOptions = options.dataSource[dataSourceName];
                if (dataSourceOptions['entities']) {
                    const entities = new Set();
                    // loop entities and glob files to model
                    for (const entity of dataSourceOptions['entities']) {
                        if (typeof entity === 'string') {
                            // string will be glob file
                            const models = globModels(entity, appDir);
                            for (const model of models) {
                                entities.add(model);
                                this.modelMapping.set(model, dataSourceName);
                            }
                        }
                        else {
                            // model will be add to array
                            entities.add(entity);
                            this.modelMapping.set(entity, dataSourceName);
                        }
                    }
                    dataSourceOptions['entities'] = Array.from(entities);
                }
                // create data source
                await this.createInstance(dataSourceOptions, dataSourceName);
            }
        }
        else {
            throw new error_1.MidwayParameterError('DataSourceManager must set options.dataSource.');
        }
    }
    /**
     * get a data source instance
     * @param dataSourceName
     */
    getDataSource(dataSourceName) {
        return this.dataSource.get(dataSourceName);
    }
    /**
     * check data source has exists
     * @param dataSourceName
     */
    hasDataSource(dataSourceName) {
        return this.dataSource.has(dataSourceName);
    }
    getDataSourceNames() {
        return Array.from(this.dataSource.keys());
    }
    /**
     * check the data source is connected
     * @param dataSourceName
     */
    async isConnected(dataSourceName) {
        return this.checkConnected(this.getDataSource(dataSourceName));
    }
    async createInstance(config, clientName) {
        // options.default will be merge in to options.clients[id]
        config = (0, extend_1.extend)(true, {}, this.options['default'], config);
        const client = await this.createDataSource(config, clientName);
        if (client) {
            if (clientName) {
                this.dataSource.set(clientName, client);
            }
            return client;
        }
    }
    /**
     * get data source name by model or repository
     * @param modelOrRepository
     */
    getDataSourceNameByModel(modelOrRepository) {
        return this.modelMapping.get(modelOrRepository);
    }
    async stop() {
        const arr = Array.from(this.dataSource.values());
        await Promise.all(arr.map(dbh => {
            return this.destroyDataSource(dbh);
        }));
        this.dataSource.clear();
    }
}
exports.DataSourceManager = DataSourceManager;
function globModels(globString, appDir) {
    const cwd = (0, path_1.join)(appDir, globString);
    const models = [];
    // string will be glob file
    const files = (0, glob_1.run)(DEFAULT_PATTERN, {
        cwd,
    });
    for (const file of files) {
        const exports = require(file);
        if (decorator_1.Types.isClass(exports)) {
            models.push(exports);
        }
        else {
            for (const m in exports) {
                const module = exports[m];
                if (decorator_1.Types.isClass(module)) {
                    models.push(module);
                }
            }
        }
    }
    return models;
}
exports.globModels = globModels;
//# sourceMappingURL=dataSourceManager.js.map